from prowler.lib.check.models import Check, Check_Report_AWS
from prowler.providers.aws.services.iam.lib.policy import is_policy_public
from prowler.providers.aws.services.opensearch.opensearch_client import (
    opensearch_client,
)


class opensearch_service_domains_not_publicly_accessible(Check):
    def execute(self):
        findings = []
        for domain in opensearch_client.opensearch_domains.values():
            report = Check_Report_AWS(self.metadata())
            report.region = domain.region
            report.resource_id = domain.name
            report.resource_arn = domain.arn
            report.resource_tags = domain.tags
            report.status = "PASS"
            report.status_extended = (
                f"Opensearch domain {domain.name} is not publicly accessible."
            )

            if domain.vpc_id:
                report.status_extended = f"Opensearch domain {domain.name} is in a VPC, then it is not publicly accessible."
            elif domain.access_policy and is_policy_public(
                domain.access_policy, opensearch_client.audited_account
            ):
                report.status = "FAIL"
                report.status_extended = f"Opensearch domain {domain.name} is publicly accessible via access policy."

            findings.append(report)

        return findings



from prowler.lib.check.models import Check, Check_Report_AWS
from prowler.providers.aws.services.eks.eks_client import eks_client


class eks_cluster_uses_a_supported_version(Check):
    def execute(self) -> Check_Report_AWS:
        findings = []

        eks_cluster_oldest_version_supported = eks_client.audit_config.get(
            "eks_cluster_oldest_version_supported", "1.28"
        )
        eks_version_major, eks_version_minor = map(
            int, eks_cluster_oldest_version_supported.split(".")
        )

        for cluster in eks_client.clusters:
            report = Check_Report_AWS(self.metadata())
            report.region = cluster.region
            report.resource_id = cluster.name
            report.resource_arn = cluster.arn
            report.resource_tags = cluster.tags

            cluster_version_major, cluster_version_minor = map(
                int, cluster.version.split(".")
            )

            if (cluster_version_major < eks_version_major) or (
                cluster_version_major == eks_version_major
                and cluster_version_minor < eks_version_minor
            ):
                report.status = "FAIL"
                report.status_extended = f"EKS cluster {cluster.name} is using version {cluster.version}. It should be one of the supported versions: {eks_cluster_oldest_version_supported} or higher."
            else:
                report.status = "PASS"
                report.status_extended = f"EKS cluster {cluster.name} is using version {cluster.version} that is supported by AWS."

            findings.append(report)

        return findings



from dataclasses import dataclass

from azure.mgmt.cosmosdb import CosmosDBManagementClient
from azure.mgmt.cosmosdb.models import PrivateEndpointConnection

from prowler.lib.logger import logger
from prowler.providers.azure.azure_provider import AzureProvider
from prowler.providers.azure.lib.service.service import AzureService


class CosmosDB(AzureService):
    def __init__(self, provider: AzureProvider):
        super().__init__(CosmosDBManagementClient, provider)
        self.accounts = self._get_accounts()

    def _get_accounts(self):
        logger.info("CosmosDB - Getting accounts...")
        accounts = {}
        for subscription, client in self.clients.items():
            try:
                accounts_list = client.database_accounts.list()
                accounts.update({subscription: []})
                for account in accounts_list:
                    accounts[subscription].append(
                        Account(
                            id=account.id,
                            name=account.name,
                            kind=account.kind,
                            location=account.location,
                            type=account.type,
                            tags=account.tags,
                            is_virtual_network_filter_enabled=account.is_virtual_network_filter_enabled,
                            private_endpoint_connections=account.private_endpoint_connections,
                            disable_local_auth=account.disable_local_auth,
                        )
                    )
            except Exception as error:
                logger.error(
                    f"Subscription name: {subscription} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
                )
        return accounts


@dataclass
class Account:
    id: str
    name: str
    kind: str
    type: str
    tags: dict
    is_virtual_network_filter_enabled: bool
    location: str
    private_endpoint_connections: list[PrivateEndpointConnection] = None
    disable_local_auth: bool = False



from prowler.lib.check.models import Check, Check_Report_Kubernetes
from prowler.providers.kubernetes.services.kubelet.kubelet_client import kubelet_client


class kubelet_client_ca_file_set(Check):
    def execute(self) -> Check_Report_Kubernetes:
        findings = []
        for cm in kubelet_client.kubelet_config_maps:
            authentication = cm.kubelet_args.get("authentication")
            report = Check_Report_Kubernetes(self.metadata())
            report.namespace = cm.namespace
            report.resource_name = cm.name
            report.resource_id = cm.uid
            if not authentication:
                report.status = "MANUAL"
                report.status_extended = f"Kubelet does not have the argument `readOnlyPort` in config file {cm.name}, verify it in the node's arguments."
            else:
                report.status = "FAIL"
                report.status_extended = (
                    f"Kubelet is missing the client CA file in config file {cm.name}."
                )
                if "clientCAFile" in authentication.get("x509", {}):
                    report.status = "PASS"
                    report.status_extended = f"Kubelet has the client CA file configured appropriately in config file {cm.name}."
            findings.append(report)
        return findings



from ipaddress import ip_address, ip_network

from prowler.lib.logger import logger
from prowler.providers.aws.aws_provider import read_aws_regions_file


def check_full_service_access(service: str, policy: dict) -> bool:
    """
    check_full_service_access checks if the policy allows full access to a service.
    Args:
        service (str): The service to check.
        policy (dict): The policy to check.
    Returns:
        bool: True if the policy allows full access to the service, False otherwise.
    """

    full_access = False

    if policy:
        policy_statements = policy.get("Statement", [])

        if not isinstance(policy_statements, list):
            policy_statements = [policy["Statement"]]

        for statement in policy_statements:
            if statement.get("Effect", "") == "Allow":
                resources = statement.get("Resource", [])

                if not isinstance(resources, list):
                    resources = [statement.get("Resource", [])]

                if "*" in resources:
                    if "Action" in statement:
                        actions = statement.get("Action", [])

                        if not isinstance(actions, list):
                            actions = [actions]

                        if f"{service}:*" in actions:
                            full_access = True
                            break

                    elif "NotAction" in statement:
                        not_actions = statement.get("NotAction", [])

                        if not isinstance(not_actions, list):
                            not_actions = [not_actions]

                        if f"{service}:*" not in not_actions:
                            full_access = True
                            break

    return full_access


def is_condition_restricting_from_private_ip(condition_statement: dict) -> bool:
    """Check if the policy condition is coming from a private IP address.

    Keyword arguments:
    condition_statement -- The policy condition to check. For example:
        {
            "IpAddress": {
                "aws:SourceIp": "X.X.X.X"
            }
        }
    """
    try:
        CONDITION_OPERATOR = "IpAddress"
        CONDITION_KEY = "aws:sourceip"

        is_from_private_ip = False

        if condition_statement.get(CONDITION_OPERATOR, {}):
            # We need to transform the condition_statement into lowercase
            condition_statement[CONDITION_OPERATOR] = {
                k.lower(): v for k, v in condition_statement[CONDITION_OPERATOR].items()
            }

            if condition_statement[CONDITION_OPERATOR].get(CONDITION_KEY, ""):
                if not isinstance(
                    condition_statement[CONDITION_OPERATOR][CONDITION_KEY], list
                ):
                    condition_statement[CONDITION_OPERATOR][CONDITION_KEY] = [
                        condition_statement[CONDITION_OPERATOR][CONDITION_KEY]
                    ]

                for ip in condition_statement[CONDITION_OPERATOR][CONDITION_KEY]:
                    # Select if IP address or IP network searching in the string for '/'
                    if ip == "*" or ip == "0.0.0.0/0":
                        break
                    else:
                        if "/" in ip:
                            if not ip_network(ip, strict=False).is_private:
                                break
                        else:
                            if not ip_address(ip).is_private:
                                break
                else:
                    is_from_private_ip = True

    except ValueError:
        logger.error(f"Invalid IP: {ip}")
    except Exception as error:
        logger.error(
            f"{error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
        )

    return is_from_private_ip


# TODO: Add logic for deny statements
def is_policy_public(
    policy: dict,
    source_account: str = "",
    is_cross_account_allowed=True,
    not_allowed_actions: list = [],
    check_cross_service_confused_deputy=False,
) -> bool:
    """
    Check if the policy allows public access to the resource.
    If the policy gives access to an AWS service principal is considered public if the policy is not pair with conditions since it can be invoked by AWS services in other accounts.
    Args:
        policy (dict): The AWS policy to check
        source_account (str): The account to check if the access is restricted to it, default: ""
        is_cross_account_allowed (bool): If the policy can allow cross-account access, default: True (https://docs.aws.amazon.com/IAM/latest/UserGuide/confused-deputy.html#cross-service-confused-deputy-prevention)
        not_allowed_actions (list): List of actions that are not allowed, default: []. If not_allowed_actions is empty, the function will not consider the actions in the policy.
        check_cross_service_confused_deputy (bool): If the policy is checked for cross-service confused deputy, default: False
    Returns:
        bool: True if the policy allows public access, False otherwise
    """
    is_public = False
    if policy:
        for statement in policy.get("Statement", []):
            # Only check allow statements
            if statement["Effect"] == "Allow":
                principal = statement.get("Principal", "")
                if (
                    "*" in principal
                    or "arn:aws:iam::*:root" in principal
                    or (
                        isinstance(principal, dict)
                        and (
                            "*" in principal.get("AWS", "")
                            or "arn:aws:iam::*:root" in principal.get("AWS", "")
                            or (
                                isinstance(principal.get("AWS"), str)
                                and source_account
                                and not is_cross_account_allowed
                                and source_account not in principal.get("AWS", "")
                            )
                            or (
                                isinstance(principal.get("AWS"), list)
                                and (
                                    "*" in principal["AWS"]
                                    or "arn:aws:iam::*:root" in principal["AWS"]
                                    or (
                                        source_account
                                        and not is_cross_account_allowed
                                        and not any(
                                            source_account in principal_aws
                                            for principal_aws in principal["AWS"]
                                        )
                                    )
                                )
                            )
                            or "*" in principal.get("CanonicalUser", "")
                            or "arn:aws:iam::*:root"
                            in principal.get("CanonicalUser", "")
                            or check_cross_service_confused_deputy
                            and (
                                # Check if function can be invoked by other AWS services if check_cross_service_confused_deputy is True
                                (
                                    ".amazonaws.com" in principal.get("Service", "")
                                    or ".amazon.com" in principal.get("Service", "")
                                    or "*" in principal.get("Service", "")
                                )
                                and (
                                    "secretsmanager.amazonaws.com"
                                    not in principal.get(
                                        "Service", ""
                                    )  # AWS ensures that resources called by SecretsManager are executed in the same AWS account
                                    or "eks.amazonaws.com"
                                    not in principal.get(
                                        "Service", ""
                                    )  # AWS ensures that resources called by EKS are executed in the same AWS account
                                )
                            )
                        )
                    )
                ) and (
                    not not_allowed_actions  # If not_allowed_actions is empty, the function will not consider the actions in the policy
                    or (
                        statement.get(
                            "Action"
                        )  # If the statement has no action, it is not public
                        and (
                            (
                                (
                                    isinstance(statement.get("Action", ""), list)
                                    and "*" in statement["Action"]
                                )
                                or (
                                    isinstance(statement.get("Action", ""), str)
                                    and statement.get("Action", "") == "*"
                                )
                            )
                            or (
                                isinstance(statement.get("Action", ""), list)
                                and any(
                                    action in not_allowed_actions
                                    for action in statement["Action"]
                                )
                            )
                            or (statement.get("Action", "") in not_allowed_actions)
                        )
                    )
                ):
                    is_public = not statement.get("Condition", {}) or (
                        not is_condition_block_restrictive(
                            statement.get("Condition", {}),
                            source_account,
                            is_cross_account_allowed,
                        )
                        and not is_condition_block_restrictive_organization(
                            statement.get("Condition", {})
                        )
                        and not is_condition_restricting_from_private_ip(
                            statement.get("Condition", {})
                        )
                    )
                    if is_public:
                        break
    return is_public


def is_condition_block_restrictive(
    condition_statement: dict,
    source_account: str = "",
    is_cross_account_allowed=False,
):
    """
    is_condition_block_restrictive parses the IAM Condition policy block and, by default, returns True if the source_account passed as argument is within, False if not.

    If argument is_cross_account_allowed is True it tests if the Condition block includes any of the operators allowlisted returning True if does, False if not.

    Args:
        condition_statement: dict with an IAM Condition block, e.g.:
        {
            "StringLike": {
                "AWS:SourceAccount": 111122223333
            }
        }

        source_account: str with a 12-digit AWS Account number, e.g.: 111122223333, default: ""

        is_cross_account_allowed: bool to allow cross-account access, e.g.: True, default: False

    """
    is_condition_valid = False

    # The conditions must be defined in lowercase since the context key names are not case-sensitive.
    # For example, including the aws:SourceAccount context key is equivalent to testing for AWS:SourceAccount
    # https://docs.aws.amazon.com/IAM/latest/UserGuide/reference_policies_elements_condition.html
    valid_condition_options = {
        "StringEquals": [
            "aws:sourceaccount",
            "aws:sourceowner",
            "aws:sourcearn",
            "aws:principalaccount",
            "aws:principalarn",
            "aws:principalorgid",
            "aws:principalorgpaths",
            "aws:resourceaccount",
            "aws:sourcevpc",
            "aws:sourcevpce",
            "aws:sourceorgid",
            "aws:sourceorgpaths",
            "aws:userid",
            "aws:username",
            "s3:resourceaccount",
            "lambda:eventsourcetoken",  # For Alexa Home functions, a token that the invoker must supply.
        ],
        "StringLike": [
            "aws:sourceaccount",
            "aws:sourceowner",
            "aws:sourcearn",
            "aws:principalaccount",
            "aws:principalarn",
            "aws:principalorgid",
            "aws:principalorgpaths",
            "aws:resourceaccount",
            "aws:sourcevpc",
            "aws:sourcevpce",
            "aws:sourceorgid",
            "aws:sourceorgpaths",
            "aws:userid",
            "aws:username",
            "s3:resourceaccount",
            "lambda:eventsourcetoken",
        ],
        "ArnLike": ["aws:sourcearn", "aws:principalarn"],
        "ArnEquals": ["aws:sourcearn", "aws:principalarn"],
    }

    for condition_operator, condition_operator_key in valid_condition_options.items():
        if condition_operator in condition_statement:
            for value in condition_operator_key:
                # We need to transform the condition_statement into lowercase
                condition_statement[condition_operator] = {
                    k.lower(): v
                    for k, v in condition_statement[condition_operator].items()
                }

                if value in condition_statement[condition_operator]:
                    # values are a list
                    if isinstance(
                        condition_statement[condition_operator][value],
                        list,
                    ):
                        is_condition_key_restrictive = True
                        # if cross account is not allowed check for each condition block looking for accounts
                        # different than default
                        if not is_cross_account_allowed:
                            # if there is an arn/account without the source account -> we do not consider it safe
                            # here by default we assume is true and look for false entries
                            for item in condition_statement[condition_operator][value]:
                                if (
                                    "aws:sourcevpc" != value
                                    and "aws:sourcevpce" != value
                                ):
                                    if source_account not in item:
                                        is_condition_key_restrictive = False
                                        break

                        if is_condition_key_restrictive:
                            is_condition_valid = True

                    # value is a string
                    elif isinstance(
                        condition_statement[condition_operator][value],
                        str,
                    ):
                        if "aws:sourcevpc" == value or "aws:sourcevpce" == value:
                            is_condition_valid = True
                        else:
                            if is_cross_account_allowed:
                                is_condition_valid = True
                            else:
                                if (
                                    source_account
                                    in condition_statement[condition_operator][value]
                                ):
                                    is_condition_valid = True

    return is_condition_valid


def is_condition_block_restrictive_organization(
    condition_statement: dict,
):
    """
    is_condition_block_restrictive_organization parses the IAM Condition policy block and returns True if the condition_statement is restrictive for the organization, False if not.

    @param condition_statement: dict with an IAM Condition block, e.g.:
        {
            "StringLike": {
                "AWS:PrincipalOrgID": "o-111122223333"
            }
        }

    """
    is_condition_valid = False

    # The conditions must be defined in lowercase since the context key names are not case-sensitive.
    # For example, including the aws:PrincipalOrgID context key is equivalent to testing for AWS:PrincipalOrgID
    # https://docs.aws.amazon.com/IAM/latest/UserGuide/reference_policies_elements_condition.html
    valid_condition_options = {
        "StringEquals": [
            "aws:principalorgid",
        ],
        "StringLike": [
            "aws:principalorgid",
        ],
    }

    for condition_operator, condition_operator_key in valid_condition_options.items():
        if condition_operator in condition_statement:
            for value in condition_operator_key:
                # We need to transform the condition_statement into lowercase
                condition_statement[condition_operator] = {
                    k.lower(): v
                    for k, v in condition_statement[condition_operator].items()
                }

                if value in condition_statement[condition_operator]:
                    # values are a list
                    if isinstance(
                        condition_statement[condition_operator][value],
                        list,
                    ):
                        is_condition_valid = True
                        for item in condition_statement[condition_operator][value]:
                            if item == "*":
                                is_condition_valid = False
                                break

                    # value is a string
                    elif isinstance(
                        condition_statement[condition_operator][value],
                        str,
                    ):
                        if "*" not in condition_statement[condition_operator][value]:
                            is_condition_valid = True

    return is_condition_valid


def process_actions(effect, actions, target_set):
    """
    process_actions processes the actions in the policy.
    Args:
        effect (str): The effect of the policy.
        actions (str or list): The actions to process.
        target_set (set): The set to store the actions.
    """
    if effect in ["Allow", "Deny"] and actions:
        if isinstance(actions, str):
            target_set.add(actions)
        elif isinstance(actions, list):
            target_set.update(actions)


def check_admin_access(policy: dict) -> bool:
    """
    check_admin_access checks if the policy allows admin access.
    Args:
        policy (dict): The policy to check.
    Returns:
        bool: True if the policy allows admin access, False otherwise.
    """

    if policy:
        allowed_actions = set()
        allowed_not_actions = set()
        denied_actions = set()
        denied_not_actions = set()

        statements = policy.get("Statement", [])
        if not isinstance(statements, list):
            statements = [statements]

        for statement in statements:
            if statement.get("Resource") in [
                "*",
                ["*"],
                ["*/*"],
                "*/*",
                ["*:*"],
                "*:*",
            ] or (
                statement.get("NotResource")
                and statement.get("NotResource") not in ["*", ["*"]]
            ):
                effect = statement.get("Effect")
                actions = statement.get("Action")
                not_actions = statement.get("NotAction")
                if effect == "Allow":
                    process_actions(effect, actions, allowed_actions)
                    process_actions(effect, not_actions, allowed_not_actions)
                elif effect == "Deny":
                    process_actions(effect, actions, denied_actions)
                    process_actions(effect, not_actions, denied_not_actions)

        # If there is only NotAction, it allows the rest of the actions
        if not allowed_actions and allowed_not_actions:
            allowed_actions.add("*")
        # Check for invalid services in allowed NotAction
        if allowed_not_actions:
            invalid_not_actions = check_invalid_not_actions(allowed_not_actions)
            if invalid_not_actions:
                # Since it is an invalid NotAction, it allows all AWS actions
                allowed_actions.add("*")

        if "*" in allowed_actions:
            return True
        return False


def check_invalid_not_actions(not_actions):
    """
    Checks if the actions in NotAction have services that are not part of AWS.
    Args:
        not_actions (str or list): The NotAction to check.
    Returns:
        dict: A dictionary with invalid services and their actions.
    """
    invalid_services = {}

    if isinstance(not_actions, str):
        not_actions = [not_actions]

    for action in not_actions:
        service = action.split(":")[0]
        if not is_valid_aws_service(service):
            if service not in invalid_services:
                invalid_services[service] = []
            invalid_services[service].append(action)

    return invalid_services


def is_valid_aws_service(service):
    """
    Checks if a service is a valid AWS service using aws_regions_by_service.json.
    Args:
        service (str): The service to check.
    Returns:
        bool: True if the service is valid, False otherwise.
    """
    if service in read_aws_regions_file()["services"]:
        return True
    return False



from prowler.lib.check.models import Check, Check_Report_AWS
from prowler.providers.aws.services.waf.wafregional_client import wafregional_client


class waf_regional_webacl_with_rules(Check):
    def execute(self):
        findings = []
        for acl in wafregional_client.web_acls.values():
            report = Check_Report_AWS(self.metadata())
            report.region = acl.region
            report.resource_id = acl.id
            report.resource_arn = acl.arn
            report.resource_tags = acl.tags
            report.status = "FAIL"
            report.status_extended = f"AWS WAF Regional Web ACL {acl.name} does not have any rules or rule groups."

            if acl.rules or acl.rule_groups:
                report.status = "PASS"
                report.status_extended = f"AWS WAF Regional Web ACL {acl.name} has at least one rule or rule group."

            findings.append(report)

        return findings



from prowler.lib.check.models import Check, Check_Report_AWS
from prowler.providers.aws.services.ecs.ecs_client import ecs_client


class ecs_task_definitions_no_privileged_containers(Check):
    def execute(self):
        findings = []
        for task_definition in ecs_client.task_definitions.values():
            report = Check_Report_AWS(self.metadata())
            report.region = task_definition.region
            report.resource_id = f"{task_definition.name}:{task_definition.revision}"
            report.resource_arn = task_definition.arn
            report.resource_tags = task_definition.tags
            report.status = "PASS"
            report.status_extended = f"ECS task definition {task_definition.name} with revision {task_definition.revision} does not have privileged containers."
            failed_containers = []
            for container in task_definition.container_definitions:
                if container.privileged:
                    report.status = "FAIL"
                    failed_containers.append(container.name)

            if failed_containers:
                report.status_extended = f"ECS task definition {task_definition.name} with revision {task_definition.revision} has privileged containers: {', '.join(failed_containers)}"
            findings.append(report)
        return findings



from prowler.lib.check.models import Check, Check_Report_Kubernetes
from prowler.providers.kubernetes.services.apiserver.apiserver_client import (
    apiserver_client,
)


class apiserver_kubelet_tls_auth(Check):
    def execute(self) -> Check_Report_Kubernetes:
        findings = []
        for pod in apiserver_client.apiserver_pods:
            report = Check_Report_Kubernetes(self.metadata())
            report.namespace = pod.namespace
            report.resource_name = pod.name
            report.resource_id = pod.uid
            report.status = "PASS"
            report.status_extended = f"API Server has appropriate kubelet TLS authentication configured in pod {pod.name}."
            for container in pod.containers.values():
                if "--kubelet-client-certificate" not in str(
                    container.command
                ) and "--kubelet-client-key" not in str(container.command):

                    report.status = "FAIL"
                    report.status_extended = f"API Server is missing kubelet TLS authentication arguments in pod {pod.name}."
                    break
            findings.append(report)
        return findings



import tempfile
from os import path
from tempfile import NamedTemporaryFile

from boto3 import Session
from botocore import exceptions

from prowler.lib.logger import logger
from prowler.lib.outputs.output import Output
from prowler.providers.aws.lib.s3.exceptions.exceptions import (
    S3BucketAccessDeniedError,
    S3ClientError,
    S3IllegalLocationConstraintError,
    S3InvalidBucketNameError,
    S3TestConnectionError,
)
from prowler.providers.common.models import Connection


class S3:
    """
    A class representing an S3 bucket.

    Attributes:
    - _session: An instance of the `Session` class representing the AWS session.
    - _bucket_name: A string representing the name of the S3 bucket.
    - _output_directory: A string representing the output directory path.

    Methods:
    - __init__: Initializes a new instance of the `S3` class.
    - get_object_path: Returns the object path within the S3 bucket based on the provided output directory.
    - generate_subfolder_name_by_extension: Generates a subfolder name based on the provided file extension.
    - send_to_bucket: Sends the provided outputs to the S3 bucket.
    """

    _session: Session
    _bucket_name: str
    _output_directory: str

    def __init__(
        self, session: Session, bucket_name: str, output_directory: str
    ) -> None:
        """
        Initializes a new instance of the `S3` class.

        Parameters:
        - session: An instance of the `Session` class representing the AWS session.
        - bucket_name: A string representing the name of the S3 bucket.
        - output_directory: A string representing the output directory path.
        """
        self._session = session.client(__class__.__name__.lower())
        self._bucket_name = bucket_name
        self._output_directory = output_directory

    @staticmethod
    def get_object_path(output_directory: str) -> str:
        """
        Return the object path within the S3 bucket based on the provided output directory.
        If the output directory contains "prowler/", it is removed to ensure the correct path is returned.

        Parameters:
        - output_directory: A string representing the output directory path.

        Returns:
        - A string representing the object path within the S3 bucket.
        """
        bucket_remote_dir = output_directory
        if "prowler/" in bucket_remote_dir:  # Check if it is not a custom directory
            bucket_remote_dir = bucket_remote_dir.partition("prowler/")[-1]

        return bucket_remote_dir

    @staticmethod
    def generate_subfolder_name_by_extension(extension: str) -> str:
        """
        Generate a subfolder name based on the provided file extension.

        Parameters:
        - extension: A string representing the file extension.

        Returns:
        - A string representing the subfolder name based on the extension.
        """
        subfolder_name = ""
        if extension == ".ocsf.json":
            subfolder_name = "json-ocsf"
        elif extension == ".asff.json":
            subfolder_name = "json-asff"
        else:
            subfolder_name = extension.lstrip(".")
        return subfolder_name

    # TODO: Review the logic behind in Microsoft Windows
    def send_to_bucket(
        self, outputs: dict[str, list[Output]]
    ) -> dict[str, dict[str, list[str]]]:
        """
        Send the provided outputs to the S3 bucket.

        Parameters:
        - outputs: A dictionary where keys are strings and values are lists of Output objects.

        Returns:
        - A dictionary containing two keys: "success" and "failure", each holding a dictionary where keys are strings and values are lists of strings representing the uploaded object names or tuples of object names and errors respectively.
        """
        try:
            uploaded_objects = {"success": {}, "failure": {}}
            # Keys are regular and/or compliance
            for key, output_list in outputs.items():
                for output in output_list:
                    try:
                        # Object is not written to file so we need to temporarily write it
                        if not hasattr(output, "file_descriptor"):
                            output.file_descriptor = NamedTemporaryFile(mode="a")

                        bucket_directory = self.get_object_path(self._output_directory)
                        basename = path.basename(output.file_descriptor.name)

                        if key == "compliance":
                            object_name = f"{bucket_directory}/{key}/{basename}"
                        else:
                            object_name = f"{bucket_directory}/{self.generate_subfolder_name_by_extension(output.file_extension)}/{basename}"
                        logger.info(
                            f"Sending output file {output.file_descriptor.name} to S3 bucket {self._bucket_name}"
                        )

                        # TODO: This will need further optimization if some processes are calling this since the files are written
                        # into the local filesystem because S3 upload file is the recommended way.
                        # https://aws.amazon.com/blogs/developer/uploading-files-to-amazon-s3/
                        self._session.upload_file(
                            output.file_descriptor.name, self._bucket_name, object_name
                        )

                        if output.file_extension in uploaded_objects["success"]:
                            uploaded_objects["success"][output.file_extension].append(
                                object_name
                            )
                        else:
                            uploaded_objects["success"] = {
                                output.file_extension: [object_name]
                            }
                    except Exception as error:
                        logger.error(
                            f"{error.__class__.__name__}[{error.__traceback__.tb_lineno}] -- {error}"
                        )
                        if output.file_extension in uploaded_objects["failure"]:
                            uploaded_objects["failure"][output.file_extension].append(
                                (object_name, error)
                            )
                        else:
                            uploaded_objects["failure"] = {
                                output.file_extension: [(object_name, error)]
                            }

        except Exception as error:
            logger.error(
                f"{error.__class__.__name__}[{error.__traceback__.tb_lineno}] -- {error}"
            )
        return uploaded_objects

    @staticmethod
    def test_connection(
        session, bucket_name: str, raise_on_exception: bool = True
    ) -> Connection:
        """
        Test the connection to the S3 bucket.

        Parameters:
        - session: An instance of the `Session` class representing the AWS session.
        - bucket_name: A string representing the name of the S3 bucket.
        - raise_on_exception: A boolean indicating whether to raise an exception if the connection test fails.

        Returns:
        - A Connection object indicating the status of the connection test.

        Raises:
        - Exception: An exception indicating that the connection test failed.
        """
        try:
            s3_client = session.client(__class__.__name__.lower())
            if "s3://" in bucket_name:
                bucket_name = bucket_name.removeprefix("s3://")
            # Check for the bucket location
            bucket_location = s3_client.get_bucket_location(Bucket=bucket_name)
            if bucket_location["LocationConstraint"] == "EU":
                bucket_location["LocationConstraint"] = "eu-west-1"
            if (
                bucket_location["LocationConstraint"] == ""
                or bucket_location["LocationConstraint"] is None
            ):
                bucket_location["LocationConstraint"] = "us-east-1"

            # If the bucket location is not the same as the session region, change the session region
            if (
                session.region_name != bucket_location["LocationConstraint"]
                and bucket_location["LocationConstraint"] is not None
            ):
                s3_client = session.client(
                    __class__.__name__.lower(),
                    region_name=bucket_location["LocationConstraint"],
                )
            # Set a Temp file to upload
            with tempfile.TemporaryFile() as temp_file:
                temp_file.write(b"Test Prowler Connection")
                temp_file.seek(0)
                s3_client.upload_fileobj(
                    temp_file, bucket_name, "test-prowler-connection.txt"
                )

            # Try to delete the file
            s3_client.delete_object(
                Bucket=bucket_name, Key="test-prowler-connection.txt"
            )
            return Connection(is_connected=True)

        except exceptions.ClientError as client_error:
            if raise_on_exception:
                if (
                    "specified bucket does not exist"
                    in client_error.response["Error"]["Message"]
                ):
                    raise S3InvalidBucketNameError(original_exception=client_error)
                elif (
                    "IllegalLocationConstraintException"
                    in client_error.response["Error"]["Message"]
                ):
                    raise S3IllegalLocationConstraintError(
                        original_exception=client_error
                    )
                elif "AccessDenied" in client_error.response["Error"]["Code"]:
                    raise S3BucketAccessDeniedError(original_exception=client_error)
                else:
                    raise S3ClientError(original_exception=client_error)
            return Connection(is_connected=False, error=client_error)

        except Exception as error:
            if raise_on_exception:
                raise S3TestConnectionError(original_exception=error)
            return False



from prowler.lib.check.models import Check, Check_Report_AWS
from prowler.providers.aws.services.ec2.ec2_client import ec2_client
from prowler.providers.aws.services.ec2.lib.network_acls import check_network_acl


class ec2_networkacl_allow_ingress_tcp_port_3389(Check):
    def execute(self):
        findings = []
        tcp_protocol = "6"
        check_port = 3389
        for arn, network_acl in ec2_client.network_acls.items():
            if (
                ec2_client.provider.scan_unused_services
                or network_acl.region in ec2_client.regions_with_sgs
            ):
                # If some entry allows it, that ACL is not securely configured
                if check_network_acl(network_acl.entries, tcp_protocol, check_port):
                    report = Check_Report_AWS(self.metadata())
                    report.resource_id = network_acl.id
                    report.region = network_acl.region
                    report.resource_arn = arn
                    report.resource_tags = network_acl.tags
                    report.status = "FAIL"
                    report.status_extended = f"Network ACL {network_acl.name if network_acl.name else network_acl.id} has Microsoft RDP port 3389 open to the Internet."
                    findings.append(report)
                else:
                    report = Check_Report_AWS(self.metadata())
                    report.resource_id = network_acl.id
                    report.region = network_acl.region
                    report.resource_arn = network_acl.arn
                    report.resource_tags = network_acl.tags
                    report.status = "PASS"
                    report.status_extended = f"Network ACL {network_acl.name if network_acl.name else network_acl.id} does not have Microsoft RDP port 3389 open to the Internet."
                    findings.append(report)

        return findings



from prowler.lib.check.models import Check, Check_Report_AWS
from prowler.providers.aws.services.eventbridge.eventbridge_client import (
    eventbridge_client,
)
from prowler.providers.aws.services.iam.lib.policy import is_policy_public


class eventbridge_bus_cross_account_access(Check):
    def execute(self):
        findings = []
        for bus in eventbridge_client.buses.values():
            report = Check_Report_AWS(self.metadata())
            report.resource_id = bus.name
            report.resource_arn = bus.arn
            report.resource_tags = bus.tags
            report.region = bus.region
            report.status = "PASS"
            report.status_extended = (
                f"EventBridge event bus {bus.name} does not allow cross-account access."
            )
            if is_policy_public(
                bus.policy,
                eventbridge_client.audited_account,
                is_cross_account_allowed=False,
            ):
                report.status = "FAIL"
                report.status_extended = (
                    f"EventBridge event bus {bus.name} allows cross-account access."
                )

            findings.append(report)

        return findings



from prowler.lib.check.models import Check, Check_Report_AWS
from prowler.providers.aws.services.dynamodb.dynamodb_client import dynamodb_client


class dynamodb_tables_kms_cmk_encryption_enabled(Check):
    def execute(self):
        findings = []
        for table_arn, table in dynamodb_client.tables.items():
            report = Check_Report_AWS(self.metadata())
            report.resource_id = table.name
            report.resource_arn = table_arn
            report.resource_tags = table.tags
            report.region = table.region
            report.status = "FAIL"
            report.status_extended = (
                f"DynamoDB table {table.name} is using DEFAULT encryption."
            )
            if table.encryption_type == "KMS":
                report.status = "PASS"
                report.status_extended = f"DynamoDB table {table.name} has KMS encryption enabled with key {table.kms_arn.split('/')[1]}."
            findings.append(report)
        return findings



from prowler.providers.azure.services.appinsights.appinsights_service import AppInsights
from prowler.providers.common.provider import Provider

appinsights_client = AppInsights(Provider.get_global_provider())



from typing import List

from dash import html


def create_provider_card(
    provider: str, provider_logo: str, account_type: str, filtered_data
) -> List[html.Div]:
    """
    Card to display the provider's name and icon.
    Args:
        provider (str): Name of the provider.
        provider_icon (str): Icon of the provider.
    Returns:
        html.Div: Card to display the provider's name and icon.
    """
    accounts = len(
        filtered_data[filtered_data["PROVIDER"] == provider]["ACCOUNT_UID"].unique()
    )
    checks_executed = len(
        filtered_data[filtered_data["PROVIDER"] == provider]["CHECK_ID"].unique()
    )
    fails = len(
        filtered_data[
            (filtered_data["PROVIDER"] == provider)
            & (filtered_data["STATUS"] == "FAIL")
        ]
    )
    passes = len(
        filtered_data[
            (filtered_data["PROVIDER"] == provider)
            & (filtered_data["STATUS"] == "PASS")
        ]
    )
    # Take the values in the MUTED colum that are true for the provider
    if "MUTED" in filtered_data.columns:
        muted = len(
            filtered_data[
                (filtered_data["PROVIDER"] == provider)
                & (filtered_data["MUTED"] == "True")
            ]
        )
    else:
        muted = 0

    return [
        html.Div(
            [
                html.Div(
                    [
                        html.Div(
                            [
                                html.Div(
                                    [
                                        html.Div([provider_logo], className="w-8"),
                                    ],
                                    className="p-2 shadow-box-up rounded-full",
                                ),
                                html.H5(
                                    f"{provider.upper()} {account_type}",
                                    className="text-base font-semibold leading-snug tracking-normal text-gray-900",
                                ),
                            ],
                            className="flex justify-between items-center mb-3",
                        ),
                        html.Div(
                            [
                                html.Div(
                                    [
                                        html.Span(
                                            account_type,
                                            className="text-prowler-stone-900 inline-block text-3xs font-bold uppercase transition-all rounded-lg text-prowler-stone-900 shadow-box-up px-4 py-1 text-center col-span-6 flex justify-center items-center",
                                        ),
                                        html.Div(
                                            accounts,
                                            className="inline-block text-xs  text-prowler-stone-900 font-bold shadow-box-down px-4 py-1 rounded-lg text-center col-span-5 col-end-13",
                                        ),
                                    ],
                                    className="grid grid-cols-12",
                                ),
                                html.Div(
                                    [
                                        html.Span(
                                            "Checks",
                                            className="text-prowler-stone-900 inline-block text-3xs font-bold uppercase transition-all rounded-lg text-prowler-stone-900 shadow-box-up px-4 py-1 text-center col-span-6 flex justify-center items-center",
                                        ),
                                        html.Div(
                                            checks_executed,
                                            className="inline-block text-xs  text-prowler-stone-900 font-bold shadow-box-down px-4 py-1 rounded-lg text-center col-span-5 col-end-13",
                                        ),
                                    ],
                                    className="grid grid-cols-12",
                                ),
                                html.Div(
                                    [
                                        html.Span(
                                            "FAILED",
                                            className="text-prowler-stone-900 inline-block text-3xs font-bold uppercase transition-all rounded-lg text-prowler-stone-900 shadow-box-up px-4 py-1 text-center col-span-6 flex justify-center items-center",
                                        ),
                                        html.Div(
                                            [
                                                html.Div(
                                                    fails,
                                                    className="m-[2px] px-4 py-1 rounded-lg bg-gradient-failed",
                                                ),
                                            ],
                                            className="inline-block text-xs font-bold shadow-box-down  rounded-lg text-center col-span-5 col-end-13",
                                        ),
                                    ],
                                    className="grid grid-cols-12",
                                ),
                                html.Div(
                                    [
                                        html.Span(
                                            "PASSED",
                                            className="text-prowler-stone-900 inline-block text-3xs font-bold uppercase transition-all rounded-lg text-prowler-stone-900 shadow-box-up px-4 py-1 text-center col-span-6 flex justify-center items-center",
                                        ),
                                        html.Div(
                                            [
                                                html.Div(
                                                    passes,
                                                    className="m-[2px] px-4 py-1 rounded-lg bg-gradient-passed",
                                                ),
                                            ],
                                            className="inline-block text-xs font-bold shadow-box-down rounded-lg text-center col-span-5 col-end-13",
                                        ),
                                    ],
                                    className="grid grid-cols-12",
                                ),
                                html.Div(
                                    [
                                        html.Span(
                                            "MUTED",
                                            className="text-prowler-stone-900 inline-block text-3xs font-bold uppercase transition-all rounded-lg text-prowler-stone-900 shadow-box-up px-4 py-1 text-center col-span-6 flex justify-center items-center",
                                        ),
                                        html.Div(
                                            [
                                                html.Div(
                                                    muted,
                                                    className="m-[2px] px-4 py-1 rounded-lg bg-gradient-muted",
                                                ),
                                            ],
                                            className="inline-block text-xs font-bold shadow-box-down rounded-lg text-center col-span-5 col-end-13",
                                        ),
                                    ],
                                    className="grid grid-cols-12",
                                ),
                            ],
                            className="grid gap-x-8 gap-y-4",
                        ),
                    ],
                    className="px-4 py-3",
                ),
            ],
            className="relative flex flex-col bg-white shadow-provider rounded-xl w-full transition ease-in-out delay-100 hover:-translate-y-1 hover:scale-110 hover:z-50 hover:cursor-pointer",
        )
    ]



from dataclasses import dataclass

from kubernetes import client

from prowler.config.config import output_file_timestamp
from prowler.providers.common.models import ProviderOutputOptions


@dataclass
class KubernetesIdentityInfo:
    context: str
    cluster: str
    user: str


@dataclass
class KubernetesSession:
    """
    KubernetesSession stores the Kubernetes session's configuration.

    """

    api_client: client.ApiClient
    context: dict


class KubernetesOutputOptions(ProviderOutputOptions):
    def __init__(self, arguments, bulk_checks_metadata, identity):
        # First call ProviderOutputOptions init
        super().__init__(arguments, bulk_checks_metadata)
        # TODO move the below if to ProviderOutputOptions
        # Check if custom output filename was input, if not, set the default
        if (
            not hasattr(arguments, "output_filename")
            or arguments.output_filename is None
        ):
            self.output_filename = f"prowler-output-{identity.context.replace(':', '_').replace('/', '_')}-{output_file_timestamp}"
        else:
            self.output_filename = arguments.output_filename



from typing import List

from prowler.lib.check.models import Check, Check_Report_AWS
from prowler.providers.aws.services.dms.dms_client import dms_client


class dms_endpoint_redis_in_transit_encryption_enabled(Check):
    """
    Check if AWS DMS Endpoints for Redis OSS have TLS enabled.

    This class verifies whether each AWS DMS Endpoint configured for Redis OSS is encrypted in transit
    by checking the `TlsEnabled` property in the endpoint's configuration. The check ensures that
    TLS is enabled to secure data in transit, preventing unauthorized access and ensuring data integrity.
    """

    def execute(self) -> List[Check_Report_AWS]:
        """
        Execute the DMS Redis TLS enabled check.

        Iterates over all DMS Endpoints and generates a report indicating whether
        each Redis OSS endpoint is encrypted in transit.

        Returns:
            List[Check_Report_AWS]: A list of report objects with the results of the check.
        """
        findings = []
        for endpoint_arn, endpoint in dms_client.endpoints.items():
            if endpoint.engine_name == "redis":
                report = Check_Report_AWS(self.metadata())
                report.resource_id = endpoint.id
                report.resource_arn = endpoint_arn
                report.region = endpoint.region
                report.resource_tags = endpoint.tags
                report.status = "FAIL"
                report.status_extended = f"DMS Endpoint {endpoint.id} for Redis OSS is not encrypted in transit."
                if endpoint.redis_ssl_protocol == "ssl-encryption":
                    report.status = "PASS"
                    report.status_extended = f"DMS Endpoint {endpoint.id} for Redis OSS is encrypted in transit."

                findings.append(report)

        return findings



from prowler.lib.check.models import Check, Check_Report_AWS
from prowler.lib.logger import logger
from prowler.providers.aws.services.iam.iam_client import iam_client


class iam_user_two_active_access_key(Check):
    def execute(self) -> Check_Report_AWS:
        try:
            findings = []
            response = iam_client.credential_report
            for user in response:
                report = Check_Report_AWS(self.metadata())
                report.resource_id = user["user"]
                report.resource_arn = user["arn"]
                report.region = iam_client.region
                # Search user in iam_client.users to get tags
                for iam_user in iam_client.users:
                    if iam_user.arn == user["arn"]:
                        report.resource_tags = iam_user.tags
                        break
                if (
                    user["access_key_1_active"] == "true"
                    and user["access_key_2_active"] == "true"
                ):
                    report.status = "FAIL"
                    report.status_extended = (
                        f"User {user['user']} has 2 active access keys."
                    )
                else:
                    report.status = "PASS"
                    report.status_extended = (
                        f"User {user['user']} does not have 2 active access keys."
                    )
                findings.append(report)
        except Exception as error:
            logger.error(f"{error.__class__.__name__} -- {error}")
        finally:
            return findings



from typing import Optional

from pydantic import BaseModel

from prowler.lib.logger import logger
from prowler.lib.scan_filters.scan_filters import is_resource_filtered
from prowler.providers.aws.lib.service.service import AWSService


################################ StorageGateway
class StorageGateway(AWSService):
    def __init__(self, provider):
        # Call AWSService's __init__
        super().__init__(__class__.__name__, provider)
        self.fileshares = []
        self.gateways = []
        self.__threading_call__(self._list_file_shares)
        self.__threading_call__(self._describe_nfs_file_shares)
        self.__threading_call__(self._describe_smb_file_shares)
        self.__threading_call__(self._list_gateways)

    def _list_file_shares(self, regional_client):
        logger.info("StorageGateway - List FileShares...")
        try:
            list_file_share_paginator = regional_client.get_paginator(
                "list_file_shares"
            )
            for page in list_file_share_paginator.paginate():
                for fileshare in page["FileShareInfoList"]:
                    if not self.audit_resources or (
                        is_resource_filtered(
                            fileshare["FileShareARN"], self.audit_resources
                        )
                    ):
                        self.fileshares.append(
                            FileShare(
                                id=fileshare["FileShareId"],
                                arn=fileshare["FileShareARN"],
                                gateway_arn=fileshare["GatewayARN"],
                                region=regional_client.region,
                                fs_type=fileshare["FileShareType"],
                                status=fileshare["FileShareStatus"],
                            )
                        )

        except Exception as error:
            logger.error(
                f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )

    def _describe_nfs_file_shares(self, regional_client):
        logger.info("StorageGateway - Describe NFS FileShares...")
        try:
            for fileshare in self.fileshares:
                if fileshare.fs_type == "NFS":
                    response = regional_client.describe_nfs_file_shares(
                        FileShareARNList=[fileshare.arn]
                    )
                    fileshare.tags = response["NFSFileShareInfoList"][0].get("Tags", [])
                    fileshare.kms = response["NFSFileShareInfoList"][0]["KMSEncrypted"]
                    fileshare.kms_key = response["NFSFileShareInfoList"][0].get(
                        "KMSKey", ""
                    )

        except Exception as error:
            logger.error(
                f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )

    def _describe_smb_file_shares(self, regional_client):
        logger.info("StorageGateway - Describe SMB FileShares...")
        try:
            for fileshare in self.fileshares:
                if fileshare.fs_type == "SMB":
                    response = regional_client.describe_smb_file_shares(
                        FileShareARNList=[fileshare.arn]
                    )
                    fileshare.tags = response["SMBFileShareInfoList"][0].get("Tags", [])
                    fileshare.kms = response["SMBFileShareInfoList"][0]["KMSEncrypted"]
                    fileshare.kms_key = response["SMBFileShareInfoList"][0].get(
                        "KMSKey", ""
                    )

        except Exception as error:
            logger.error(
                f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )

    def _list_gateways(self, regional_client):
        logger.info("StorageGateway - List Gateways...")
        try:
            list_gateway_paginator = regional_client.get_paginator("list_gateways")
            for page in list_gateway_paginator.paginate():
                for gateway in page["Gateways"]:
                    if not self.audit_resources or (
                        is_resource_filtered(
                            gateway["GatewayARN"], self.audit_resources
                        )
                    ):
                        self.gateways.append(
                            Gateway(
                                id=gateway["GatewayId"],
                                arn=gateway["GatewayARN"],
                                name=gateway["GatewayName"],
                                type=gateway["GatewayType"],
                                region=regional_client.region,
                                environment=gateway["HostEnvironment"],
                            )
                        )

        except Exception as error:
            logger.error(
                f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )


class FileShare(BaseModel):
    id: str
    arn: str
    gateway_arn: str
    region: str
    fs_type: str
    status: str
    kms: Optional[bool]
    kms_key: Optional[str]
    tags: Optional[list] = []


class Gateway(BaseModel):
    id: str
    arn: str
    name: str
    type: str
    region: str
    environment: str



from prowler.lib.check.models import Check, Check_Report_AWS
from prowler.providers.aws.services.networkfirewall.networkfirewall_client import (
    networkfirewall_client,
)
from prowler.providers.aws.services.vpc.vpc_client import vpc_client


class networkfirewall_in_all_vpc(Check):
    def execute(self):
        findings = []
        for vpc in vpc_client.vpcs.values():
            if vpc_client.provider.scan_unused_services or vpc.in_use:
                report = Check_Report_AWS(self.metadata())
                report.region = vpc.region
                report.resource_id = vpc.id
                report.resource_arn = vpc.arn
                report.resource_tags = vpc.tags
                report.status = "FAIL"
                report.status_extended = f"VPC {vpc.name if vpc.name else vpc.id} does not have Network Firewall enabled."
                for firewall in networkfirewall_client.network_firewalls.values():
                    if firewall.vpc_id == vpc.id:
                        report.status = "PASS"
                        report.status_extended = f"VPC {vpc.name if vpc.name else vpc.id} has Network Firewall enabled."
                        break

                findings.append(report)

        return findings



import shodan

from prowler.lib.check.models import Check, Check_Report_Azure
from prowler.lib.logger import logger
from prowler.providers.azure.services.network.network_client import network_client


class network_public_ip_shodan(Check):
    def execute(self):
        findings = []
        shodan_api_key = network_client.audit_config.get("shodan_api_key")
        if shodan_api_key:
            api = shodan.Shodan(shodan_api_key)
            for subscription, public_ips in network_client.public_ip_addresses.items():
                for ip in public_ips:
                    report = Check_Report_Azure(self.metadata())
                    report.subscription = subscription
                    report.resource_name = ip.name
                    report.resource_id = ip.id
                    report.location = ip.location
                    try:
                        shodan_info = api.host(ip.ip_address)
                        report.status = "FAIL"
                        report.status_extended = f"Public IP {ip.ip_address} listed in Shodan with open ports {str(shodan_info['ports'])} and ISP {shodan_info['isp']} in {shodan_info['country_name']}. More info at https://www.shodan.io/host/{ip.ip_address}."
                        findings.append(report)
                    except shodan.APIError as error:
                        if "No information available for that IP" in error.value:
                            report.status = "PASS"
                            report.status_extended = (
                                f"Public IP {ip.ip_address} is not listed in Shodan."
                            )
                            findings.append(report)
                            continue
                        else:
                            logger.error(f"Unknown Shodan API Error: {error.value}")

        else:
            logger.error(
                "No Shodan API Key -- Please input a Shodan API Key with -N/--shodan or in config.yaml"
            )
        return findings



from prowler.lib.check.models import Check, Check_Report_Azure
from prowler.providers.azure.services.network.network_client import network_client


class network_watcher_enabled(Check):
    def execute(self) -> list[Check_Report_Azure]:
        findings = []
        for subscription, network_watchers in network_client.network_watchers.items():
            report = Check_Report_Azure(self.metadata())
            report.subscription = subscription
            report.resource_name = "Network Watcher"
            report.location = "global"
            report.resource_id = f"/subscriptions/{network_client.subscriptions[subscription]}/resourceGroups/NetworkWatcherRG/providers/Microsoft.Network/networkWatchers/NetworkWatcher_*"

            missing_locations = set(network_client.locations[subscription]) - set(
                network_watcher.location for network_watcher in network_watchers
            )

            if missing_locations:
                report.status = "FAIL"
                report.status_extended = f"Network Watcher is not enabled for the following locations in subscription '{subscription}': {', '.join(missing_locations)}."
            else:
                report.status = "PASS"
                report.status_extended = f"Network Watcher is enabled for all locations in subscription '{subscription}'."

            findings.append(report)

        return findings



from prowler.lib.check.models import Check, Check_Report_AWS
from prowler.providers.aws.services.glue.glue_client import glue_client
from prowler.providers.aws.services.iam.lib.policy import is_policy_public


class glue_data_catalogs_not_publicly_accessible(Check):
    def execute(self):
        findings = []
        for data_catalog in glue_client.data_catalogs.values():
            report = Check_Report_AWS(self.metadata())
            report.region = data_catalog.region
            report.resource_id = glue_client.audited_account
            report.resource_arn = glue_client._get_data_catalog_arn_template(
                data_catalog.region
            )
            report.status = "PASS"
            report.status_extended = "Glue Data Catalog is not publicly accessible."
            if is_policy_public(
                data_catalog.policy,
                glue_client.audited_account,
            ):
                report.status = "FAIL"
                report.status_extended = "Glue Data Catalog is publicly accessible due to its resource policy."

            findings.append(report)

        return findings



from prowler.exceptions.exceptions import ProwlerException


# Exceptions codes from 7000 to 7999 are reserved for Security Hub exceptions
class SecurityHubBaseException(ProwlerException):
    """Base class for Security Hub exceptions."""

    SECURITYHUB_ERROR_CODES = {
        (7000, "SecurityHubNoEnabledRegionsError"): {
            "message": "No regions were found to with the Security Hub integration enabled.",
            "remediation": "Please check the connection settings and permissions and try again.",
        },
        (7001, "SecurityHubInvalidRegionError"): {
            "message": "Given region has not Security Hub enabled.",
            "remediation": "Please provide a valid region.",
        },
    }

    def __init__(self, code, file=None, original_exception=None, message=None):
        module = "SecurityHub"
        error_info = self.SECURITYHUB_ERROR_CODES.get((code, self.__class__.__name__))
        if message:
            error_info["message"] = message
        super().__init__(
            code=code,
            source=module,
            file=file,
            original_exception=original_exception,
            error_info=error_info,
        )


class SecurityHubNoEnabledRegionsError(SecurityHubBaseException):
    def __init__(self, file=None, original_exception=None, message=None):
        super().__init__(
            7000, file=file, original_exception=original_exception, message=message
        )


class SecurityHubInvalidRegionError(SecurityHubBaseException):
    def __init__(self, file=None, original_exception=None, message=None):
        super().__init__(
            7001, file=file, original_exception=original_exception, message=message
        )



from prowler.lib.check.models import Check, Check_Report_Azure
from prowler.providers.azure.services.sqlserver.sqlserver_client import sqlserver_client


class sqlserver_tde_encryption_enabled(Check):
    def execute(self) -> Check_Report_Azure:
        findings = []
        for subscription, sql_servers in sqlserver_client.sql_servers.items():
            for sql_server in sql_servers:
                databases = (
                    sql_server.databases if sql_server.databases is not None else []
                )
                if len(databases) > 0:
                    for database in databases:
                        report = Check_Report_Azure(self.metadata())
                        report.subscription = subscription
                        report.resource_name = database.name
                        report.resource_id = database.id
                        report.location = sql_server.location
                        if database.tde_encryption.status == "Enabled":
                            report.status = "PASS"
                            report.status_extended = f"Database {database.name} from SQL Server {sql_server.name} from subscription {subscription} has TDE enabled"
                        else:
                            report.status = "FAIL"
                            report.status_extended = f"Database {database.name} from SQL Server {sql_server.name} from subscription {subscription} has TDE disabled"
                        findings.append(report)

        return findings



from prowler.lib.check.models import Check, Check_Report_AWS
from prowler.providers.aws.services.mq.mq_client import mq_client
from prowler.providers.aws.services.mq.mq_service import EngineType


class mq_broker_logging_enabled(Check):
    """Ensure that MQ Brokers have logging enabled

    This check will return FAIL if the MQ Broker does not have logging enabled.
    """

    def execute(self):
        """Execute the check

        Returns: List[Check_Report_AWS]: List of check reports
        """
        findings = []
        for broker in mq_client.brokers.values():
            report = Check_Report_AWS(self.metadata())
            report.region = broker.region
            report.resource_id = broker.id
            report.resource_arn = broker.arn
            report.resource_tags = broker.tags
            report.status = "FAIL"
            report.status_extended = (
                f"MQ Broker {broker.name} does not have logging enabled."
            )
            logging_enabled = False

            if broker.engine_type == EngineType.ACTIVEMQ:
                logging_enabled = (
                    broker.general_logging_enabled and broker.audit_logging_enabled
                )
            elif broker.engine_type == EngineType.RABBITMQ:
                logging_enabled = broker.general_logging_enabled

            if logging_enabled:
                report.status = "PASS"
                report.status_extended = (
                    f"MQ Broker {broker.name} does have logging enabled."
                )

            findings.append(report)

        return findings



from prowler.lib.check.models import Check, Check_Report_AWS
from prowler.providers.aws.services.kms.kms_client import kms_client


class kms_cmk_rotation_enabled(Check):
    def execute(self):
        findings = []
        for key in kms_client.keys:
            report = Check_Report_AWS(self.metadata())
            report.region = key.region
            report.resource_tags = key.tags
            # Only check enabled CMKs keys
            if (
                key.manager == "CUSTOMER"
                and key.state == "Enabled"
                and "SYMMETRIC" in key.spec
            ):
                if key.rotation_enabled:
                    report.status = "PASS"
                    report.status_extended = (
                        f"KMS CMK {key.id} has automatic rotation enabled."
                    )
                    report.resource_id = key.id
                    report.resource_arn = key.arn
                else:
                    report.status = "FAIL"
                    report.status_extended = (
                        f"KMS CMK {key.id} has automatic rotation disabled."
                    )
                    report.resource_id = key.id
                    report.resource_arn = key.arn
                findings.append(report)
        return findings



from prowler.lib.check.models import Check, Check_Report_AWS
from prowler.providers.aws.services.cognito.cognito_idp_client import cognito_idp_client


class cognito_user_pool_password_policy_minimum_length_14(Check):
    def execute(self):
        findings = []
        for pool in cognito_idp_client.user_pools.values():
            report = Check_Report_AWS(self.metadata())
            report.region = pool.region
            report.resource_id = pool.id
            report.resource_arn = pool.arn
            report.resource_tags = pool.tags
            if pool.password_policy:
                if pool.password_policy.minimum_length >= 14:
                    report.status = "PASS"
                    report.status_extended = f"User pool {pool.name} has a password policy with a minimum length of 14 characters."
                else:
                    report.status = "FAIL"
                    report.status_extended = f"User pool {pool.name} does not have a password policy with a minimum length of 14 characters."
            else:
                report.status = "FAIL"
                report.status_extended = (
                    f"User pool {pool.name} has not a password policy set."
                )
            findings.append(report)

        return findings



from prowler.lib.check.models import Check, Check_Report_AWS
from prowler.providers.aws.services.iam.iam_client import iam_client


class iam_securityaudit_role_created(Check):
    def execute(self) -> Check_Report_AWS:
        findings = []
        if iam_client.entities_role_attached_to_securityaudit_policy is not None:
            report = Check_Report_AWS(self.metadata())
            report.region = iam_client.region
            report.resource_id = "SecurityAudit"
            report.resource_arn = (
                f"arn:{iam_client.audited_partition}:iam::aws:policy/SecurityAudit"
            )
            if iam_client.entities_role_attached_to_securityaudit_policy:
                report.status = "PASS"
                report.status_extended = f"SecurityAudit policy attached to role {iam_client.entities_role_attached_to_securityaudit_policy[0]['RoleName']}."
            else:
                report.status = "FAIL"
                report.status_extended = (
                    "SecurityAudit policy is not attached to any role."
                )
            findings.append(report)
        return findings



from prowler.providers.aws.services.storagegateway.storagegateway_service import (
    StorageGateway,
)
from prowler.providers.common.provider import Provider

storagegateway_client = StorageGateway(Provider.get_global_provider())



from prowler.lib.check.models import Check, Check_Report_AWS
from prowler.providers.aws.services.cloudfront.cloudfront_client import (
    cloudfront_client,
)
from prowler.providers.aws.services.cloudfront.cloudfront_service import (
    SSLSupportMethod,
)


class cloudfront_distributions_https_sni_enabled(Check):
    def execute(self):
        findings = []
        for distribution in cloudfront_client.distributions.values():
            if distribution.certificate:
                report = Check_Report_AWS(self.metadata())
                report.region = distribution.region
                report.resource_arn = distribution.arn
                report.resource_id = distribution.id
                report.resource_tags = distribution.tags

                if distribution.ssl_support_method == SSLSupportMethod.sni_only:
                    report.status = "PASS"
                    report.status_extended = f"CloudFront Distribution {distribution.id} is serving HTTPS requests using SNI."
                else:
                    report.status = "FAIL"
                    report.status_extended = f"CloudFront Distribution {distribution.id} is not serving HTTPS requests using SNI."

                findings.append(report)

        return findings



from enum import Enum
from typing import Dict, List

from pydantic import BaseModel, Field

from prowler.lib.logger import logger
from prowler.lib.scan_filters.scan_filters import is_resource_filtered
from prowler.providers.aws.lib.service.service import AWSService


class Transfer(AWSService):
    def __init__(self, provider):
        # Call AWSService's __init__
        super().__init__(__class__.__name__, provider)
        self.servers = {}
        self.__threading_call__(self._list_servers)
        self.__threading_call__(self._describe_server, self.servers.values())

    def _list_servers(self, regional_client):
        logger.info("Transfer - Listing Transfer Servers...")
        try:
            list_servers_paginator = regional_client.get_paginator("list_servers")
            for page in list_servers_paginator.paginate():
                for server in page["Servers"]:
                    arn = server["Arn"]
                    if not self.audit_resources or (
                        is_resource_filtered(arn, self.audit_resources)
                    ):
                        self.servers[arn] = Server(
                            arn=arn,
                            id=server.get("ServerId", ""),
                            region=regional_client.region,
                        )
        except Exception as error:
            logger.error(
                f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )

    def _describe_server(self, server):
        logger.info(f"Transfer - Describing Server {server.id}...")
        try:
            server_description = (
                self.regional_clients[server.region]
                .describe_server(ServerId=server.id)
                .get("Server", {})
            )
            for protocol in server_description.get("Protocols", []):
                server.protocols.append(Protocol(protocol))
            server.tags = server_description.get("Tags", [])
        except Exception as error:
            logger.error(
                f"{server.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )


class Protocol(Enum):
    FTP = "FTP"
    FTPS = "FTPS"
    SFTP = "SFTP"
    AS2 = "AS2"


class Server(BaseModel):
    arn: str
    id: str
    region: str
    protocols: List[Protocol] = Field(default_factory=list)
    tags: List[Dict[str, str]] = Field(default_factory=list)



from prowler.lib.check.models import Check, Check_Report_Azure
from prowler.providers.azure.services.monitor.lib.monitor_alerts import check_alert_rule
from prowler.providers.azure.services.monitor.monitor_client import monitor_client


class monitor_alert_create_policy_assignment(Check):
    def execute(self) -> Check_Report_Azure:
        findings = []

        for (
            subscription_name,
            activity_log_alerts,
        ) in monitor_client.alert_rules.items():
            report = Check_Report_Azure(self.metadata())
            report.status = "FAIL"
            report.subscription = subscription_name
            report.resource_name = "Monitor"
            report.resource_id = "Monitor"
            report.status_extended = f"There is not an alert for creating Policy Assignments in subscription {subscription_name}."
            for alert_rule in activity_log_alerts:
                if check_alert_rule(
                    alert_rule, "Microsoft.Authorization/policyAssignments/write"
                ):
                    report.status = "PASS"
                    report.resource_name = alert_rule.name
                    report.resource_id = alert_rule.id
                    report.subscription = subscription_name
                    report.status_extended = f"There is an alert configured for creating Policy Assignments in subscription {subscription_name}."
                    break

            findings.append(report)

        return findings



from prowler.lib.check.models import Check, Check_Report_AWS
from prowler.providers.aws.services.neptune.neptune_client import neptune_client


class neptune_cluster_copy_tags_to_snapshots(Check):
    def execute(self):
        findings = []
        for cluster_arn, cluster in neptune_client.clusters.items():
            report = Check_Report_AWS(self.metadata())
            report.region = cluster.region
            report.resource_id = cluster.id
            report.resource_arn = cluster.arn
            report.resource_tags = cluster.tags
            report.status = "FAIL"
            report.status_extended = f"Neptune DB Cluster {cluster.id} is not configured to copy tags to snapshots."
            if cluster.copy_tags_to_snapshot:
                report.status = "PASS"
                report.status_extended = f"Neptune DB Cluster {cluster.id} is configured to copy tags to snapshots."

            findings.append(report)

        return findings



from prowler.lib.check.models import Check, Check_Report_AWS
from prowler.providers.aws.services.glue.glue_client import glue_client


class glue_data_catalogs_metadata_encryption_enabled(Check):
    def execute(self):
        findings = []
        for data_catalog in glue_client.data_catalogs.values():
            # Check only if there are Glue Tables
            if data_catalog.tables or glue_client.provider.scan_unused_services:
                report = Check_Report_AWS(self.metadata())
                report.resource_id = glue_client.audited_account
                report.resource_arn = glue_client._get_data_catalog_arn_template(
                    data_catalog.region
                )
                report.region = data_catalog.region
                report.status = "FAIL"
                report.status_extended = (
                    "Glue data catalog settings have metadata encryption disabled."
                )
                if (
                    data_catalog.encryption_settings
                    and data_catalog.encryption_settings.mode == "SSE-KMS"
                ):
                    report.status = "PASS"
                    report.status_extended = f"Glue data catalog settings have metadata encryption enabled with KMS key {data_catalog.encryption_settings.kms_id}."
                findings.append(report)
        return findings



from prowler.lib.check.models import Check, Check_Report_AWS
from prowler.providers.aws.services.s3.s3_client import s3_client


class s3_bucket_acl_prohibited(Check):
    def execute(self):
        findings = []
        for arn, bucket in s3_client.buckets.items():
            report = Check_Report_AWS(self.metadata())
            report.region = bucket.region
            report.resource_id = bucket.name
            report.resource_arn = arn
            report.resource_tags = bucket.tags
            report.status = "FAIL"
            report.status_extended = f"S3 Bucket {bucket.name} has bucket ACLs enabled."
            if bucket.ownership:
                if "BucketOwnerEnforced" in bucket.ownership:
                    report.status = "PASS"
                    report.status_extended = (
                        f"S3 Bucket {bucket.name} has bucket ACLs disabled."
                    )
            findings.append(report)

        return findings
